Setup and Config
Getting and Creating Projects
Basic Snapshotting
Branching and Merging
Sharing and Updating Projects
Inspection and Comparison
Patching
Debugging
External Systems
Server Admin
Guides
- gitattributes
- Command-line interface conventions
- Everyday Git
- Frequently Asked Questions (FAQ)
- Glossary
- Hooks
- gitignore
- gitmodules
- Revisions
- Submodules
- Tutorial
- Workflows
- All guides...
Administration
Plumbing Commands
-
2.56.0
2026-09-28
-
2.55.0
2026-06-29
-
2.54.0
2026-04-20
-
2.53.0
2026-02-02
-
2.52.0
2025-11-17
-
2.51.2
2025-10-27
-
2.51.1
2025-10-15
-
2.51.0
2025-08-18
- 2.50.1 no changes
-
2.50.0
2025-06-16
- 2.49.1 no changes
-
2.49.0
2025-03-14
- 2.48.2 no changes
-
2.48.1
2025-01-13
-
2.48.0
2025-01-10
- 2.47.3 no changes
-
2.47.2
2024-11-26
-
2.47.1
2024-11-25
-
2.47.0
2024-10-06
- 2.46.4 no changes
-
2.46.3
2024-11-26
-
2.46.2
2024-09-23
-
2.46.1
2024-09-13
-
2.46.0
2024-07-29
- 2.45.4 no changes
-
2.45.3
2024-11-26
- 2.45.1 → 2.45.2 no changes
-
2.45.0
2024-04-29
- 2.44.4 no changes
-
2.44.3
2024-11-26
- 2.44.1 → 2.44.2 no changes
-
2.44.0
2024-02-23
- 2.43.7 no changes
-
2.43.6
2024-11-26
- 2.43.2 → 2.43.5 no changes
-
2.43.1
2024-02-09
-
2.43.0
2023-11-20
-
2.42.4
2024-11-26
- 2.42.2 → 2.42.3 no changes
-
2.42.1
2023-11-02
-
2.42.0
2023-08-21
-
2.41.3
2024-11-26
- 2.41.1 → 2.41.2 no changes
-
2.41.0
2023-06-01
-
2.40.4
2024-11-26
- 2.40.1 → 2.40.3 no changes
-
2.40.0
2023-03-12
- 2.39.1 → 2.39.5 no changes
-
2.39.0
2022-12-12
- 2.38.3 → 2.38.5 no changes
-
2.38.2
2022-12-11
-
2.38.1
2022-10-07
-
2.38.0
2022-10-02
- 2.37.5 → 2.37.7 no changes
-
2.37.4
2022-10-06
- 2.37.3 no changes
-
2.37.2
2022-08-11
- 2.37.1 no changes
-
2.37.0
2022-06-27
- 2.36.4 → 2.36.6 no changes
-
2.36.3
2022-10-06
-
2.36.2
2022-06-23
- 2.36.1 no changes
-
2.36.0
2022-04-18
- 2.35.6 → 2.35.8 no changes
-
2.35.5
2022-10-06
-
2.35.4
2022-06-23
-
2.35.3
2022-04-13
-
2.35.2
2022-03-23
- 2.35.1 no changes
-
2.35.0
2022-01-24
- 2.34.6 → 2.34.8 no changes
-
2.34.5
2022-10-06
-
2.34.4
2022-06-23
-
2.34.3
2022-04-13
-
2.34.2
2022-03-23
- 2.34.1 no changes
-
2.34.0
2021-11-15
- 2.33.6 → 2.33.8 no changes
-
2.33.5
2022-10-06
-
2.33.4
2022-06-23
-
2.33.3
2022-04-13
-
2.33.2
2022-03-23
-
2.33.1
2021-10-12
-
2.33.0
2021-08-16
- 2.32.5 → 2.32.7 no changes
-
2.32.4
2022-10-06
-
2.32.3
2022-06-23
-
2.32.2
2022-04-13
-
2.32.1
2022-03-23
-
2.32.0
2021-06-06
- 2.31.6 → 2.31.8 no changes
-
2.31.5
2022-10-06
-
2.31.4
2022-06-23
-
2.31.3
2022-04-13
-
2.31.2
2022-03-23
-
2.31.1
2021-03-26
-
2.31.0
2021-03-15
- 2.30.7 → 2.30.9 no changes
-
2.30.6
2022-10-06
-
2.30.5
2022-06-23
-
2.30.4
2022-04-13
-
2.30.3
2022-03-23
- 2.30.2 no changes
-
2.30.1
2021-02-08
-
2.30.0
2020-12-27
- 2.29.1 → 2.29.3 no changes
-
2.29.0
2020-10-19
- 2.28.1 no changes
-
2.28.0
2020-07-27
- 2.27.1 no changes
-
2.27.0
2020-06-01
- 2.26.1 → 2.26.3 no changes
-
2.26.0
2020-03-22
- 2.25.3 → 2.25.5 no changes
-
2.25.2
2020-03-17
-
2.25.1
2020-02-17
-
2.25.0
2020-01-13
- 2.24.1 → 2.24.4 no changes
-
2.24.0
2019-11-04
- 2.23.1 → 2.23.4 no changes
-
2.23.0
2019-08-16
- 2.22.2 → 2.22.5 no changes
-
2.22.1
2019-08-11
-
2.22.0
2019-06-07
- 2.21.1 → 2.21.4 no changes
-
2.21.0
2019-02-24
- 2.20.1 → 2.20.5 no changes
-
2.20.0
2018-12-09
- 2.19.3 → 2.19.6 no changes
-
2.19.2
2018-11-21
- 2.19.1 no changes
-
2.19.0
2018-09-10
- 2.18.1 → 2.18.5 no changes
-
2.18.0
2018-06-21
- 2.17.1 → 2.17.6 no changes
-
2.17.0
2018-04-02
-
2.16.6
2019-12-06
-
2.15.4
2019-12-06
-
2.14.6
2019-12-06
-
2.13.7
2018-05-22
-
2.12.5
2017-09-22
-
2.11.4
2017-09-22
-
2.10.5
2017-09-22
-
2.9.5
2017-07-30
-
2.8.6
2017-07-30
-
2.7.6
2017-07-30
-
2.6.7
2017-05-05
-
2.5.6
2017-05-05
-
2.4.12
2017-05-05
-
2.3.10
2015-09-28
-
2.2.3
2015-09-04
-
2.1.4
2014-12-17
-
2.0.5
2014-12-17
SYNOPSIS
git config [<file-option>] [--type=<type>] [--fixed-value] [--show-origin] [--show-scope] [-z|--null] name [value [value-pattern]] git config [<file-option>] [--type=<type>] --add name value git config [<file-option>] [--type=<type>] [--fixed-value] --replace-all name value [value-pattern] git config [<file-option>] [--type=<type>] [--show-origin] [--show-scope] [-z|--null] [--fixed-value] --get name [value-pattern] git config [<file-option>] [--type=<type>] [--show-origin] [--show-scope] [-z|--null] [--fixed-value] --get-all name [value-pattern] git config [<file-option>] [--type=<type>] [--show-origin] [--show-scope] [-z|--null] [--fixed-value] [--name-only] --get-regexp name_regex [value-pattern] git config [<file-option>] [--type=<type>] [-z|--null] --get-urlmatch name URL git config [<file-option>] [--fixed-value] --unset name [value-pattern] git config [<file-option>] [--fixed-value] --unset-all name [value-pattern] git config [<file-option>] --rename-section old_name new_name git config [<file-option>] --remove-section name git config [<file-option>] [--show-origin] [--show-scope] [-z|--null] [--name-only] -l | --list git config [<file-option>] --get-color name [default] git config [<file-option>] --get-colorbool name [stdout-is-tty] git config [<file-option>] -e | --edit
DESCRIPTION
You can query/set/replace/unset options with this command. The name is actually the section and the key separated by a dot, and the value will be escaped.
Multiple lines can be added to an option by using the --add option.
If you want to update or unset an option which can occur on multiple
lines, a value-pattern (which is an extended regular expression,
unless the --fixed-value option is given) needs to be given. Only the
existing values that match the pattern are updated or unset. If
you want to handle the lines that do not match the pattern, just
prepend a single exclamation mark in front (see also EXAMPLES),
but note that this only works when the --fixed-value option is not
in use.
The --type=<type> option instructs git config to ensure that incoming and
outgoing values are canonicalize-able under the given <type>. If no
--type=<type> is given, no canonicalization will be performed. Callers may
unset an existing --type specifier with --no-type.
When reading, the values are read from the system, global and
repository local configuration files by default, and options
--system, --global, --local, --worktree and
--file <filename> can be used to tell the command to read from only
that location (see FILES).
When writing, the new value is written to the repository local
configuration file by default, and options --system, --global,
--worktree, --file <filename> can be used to tell the command to
write to that location (you can say --local but that is the
default).
This command will fail with non-zero status upon error. Some exit codes are:
-
The section or key is invalid (ret=1),
-
no section or name was provided (ret=2),
-
the config file is invalid (ret=3),
-
the config file cannot be written (ret=4),
-
you try to unset an option which does not exist (ret=5),
-
you try to unset/set an option for which multiple lines match (ret=5), or
-
you try to use an invalid regexp (ret=6).
On success, the command returns the exit code 0.
OPTIONS
- --replace-all
-
Default behavior is to replace at most one line. This replaces all lines matching the key (and optionally the
value-pattern). - --add
-
Adds a new line to the option without altering any existing values. This is the same as providing ^$ as the
value-patternin--replace-all. - --get
-
Get the value for a given key (optionally filtered by a regex matching the value). Returns error code 1 if the key was not found and the last value if multiple key values were found.
- --get-all
-
Like get, but returns all values for a multi-valued key.
- --get-regexp
-
Like --get-all, but interprets the name as a regular expression and writes out the key names. Regular expression matching is currently case-sensitive and done against a canonicalized version of the key in which section and variable names are lowercased, but subsection names are not.
- --get-urlmatch name URL
-
When given a two-part name section.key, the value for section.<url>.key whose <url> part matches the best to the given URL is returned (if no such key exists, the value for section.key is used as a fallback). When given just the section as name, do so for all the keys in the section and list them. Returns error code 1 if no value is found.
- --global
-
For writing options: write to global
~/.gitconfigfile rather than the repository.git/config, write to$XDG_CONFIG_HOME/git/configfile if this file exists and the~/.gitconfigfile doesn’t.For reading options: read only from global
~/.gitconfigand from$XDG_CONFIG_HOME/git/configrather than from all available files.See also FILES.
- --system
-
For writing options: write to system-wide
$(prefix)/etc/gitconfigrather than the repository.git/config.For reading options: read only from system-wide
$(prefix)/etc/gitconfigrather than from all available files.See also FILES.
- --local
-
For writing options: write to the repository
.git/configfile. This is the default behavior.For reading options: read only from the repository
.git/configrather than from all available files.See also FILES.
- --worktree
-
Similar to
--localexcept that.git/config.worktreeis read from or written to ifextensions.worktreeConfigis present. If not it’s the same as--local. - -f config-file
- --file config-file
-
Use the given config file instead of the one specified by GIT_CONFIG.
- --blob blob
-
Similar to
--filebut use the given blob instead of a file. E.g. you can use master:.gitmodules to read values from the file .gitmodules in the master branch. See "SPECIFYING REVISIONS" section in gitrevisions[7] for a more complete list of ways to spell blob names. - --remove-section
-
Remove the given section from the configuration file.
- --rename-section
-
Rename the given section to a new name.
- --unset
-
Remove the line matching the key from config file.
- --unset-all
-
Remove all lines matching the key from config file.
- -l
- --list
-
List all variables set in config file, along with their values.
- --fixed-value
-
When used with the
value-patternargument, treatvalue-patternas an exact string instead of a regular expression. This will restrict the name/value pairs that are matched to only those where the value is exactly equal to thevalue-pattern. - --type <type>
-
git config will ensure that any input or output is valid under the given type constraint(s), and will canonicalize outgoing values in <type>'s canonical form.
Valid <type>'s include:
-
bool: canonicalize values as either "true" or "false".
-
int: canonicalize values as simple decimal numbers. An optional suffix of k, m, or g will cause the value to be multiplied by 1024, 1048576, or 1073741824 upon input.
-
bool-or-int: canonicalize according to either bool or int, as described above.
-
path: canonicalize by adding a leading
~to the value of$HOMEand~userto the home directory for the specified user. This specifier has no effect when setting the value (but you can usegitconfigsection.variable~/from the command line to let your shell do the expansion.) -
expiry-date: canonicalize by converting from a fixed or relative date-string to a timestamp. This specifier has no effect when setting the value.
-
color: When getting a value, canonicalize by converting to an ANSI color escape sequence. When setting a value, a sanity-check is performed to ensure that the given value is canonicalize-able as an ANSI color, but it is written as-is.
-
- --bool
- --int
- --bool-or-int
- --path
- --expiry-date
-
Historical options for selecting a type specifier. Prefer instead
--type(see above). - --no-type
-
Un-sets the previously set type specifier (if one was previously set). This option requests that git config not canonicalize the retrieved variable.
--no-typehas no effect without--type=<type> or--<type>. - -z
- --null
-
For all options that output values and/or keys, always end values with the null character (instead of a newline). Use newline instead as a delimiter between key and value. This allows for secure parsing of the output without getting confused e.g. by values that contain line breaks.
- --name-only
-
Output only the names of config variables for
--listor--get-regexp. - --show-origin
-
Augment the output of all queried config options with the origin type (file, standard input, blob, command line) and the actual origin (config file path, ref, or blob id if applicable).
- --show-scope
-
Similar to
--show-originin that it augments the output of all queried config options with the scope of that value (local, global, system, command). - --get-colorbool name [stdout-is-tty]
-
Find the color setting for
name(e.g.color.diff) and output "true" or "false".stdout-is-ttyshould be either "true" or "false", and is taken into account when configuration says "auto". Ifstdout-is-ttyis missing, then checks the standard output of the command itself, and exits with status 0 if color is to be used, or exits with status 1 otherwise. When the color setting fornameis undefined, the command usescolor.uias fallback. - --get-color name [default]
-
Find the color configured for
name(e.g.color.diff.new) and output it as the ANSI color escape sequence to the standard output. The optionaldefaultparameter is used instead, if there is no color configured forname.--type=color[--default=<default>] is preferred over--get-color(but note that--get-colorwill omit the trailing newline printed by--type=color). - -e
- --edit
-
Opens an editor to modify the specified config file; either
--system,--global, or repository (default). - --[no-]includes
-
Respect
include.*directives in config files when looking up values. Defaults tooffwhen a specific file is given (e.g., using--file,--global, etc) andonwhen searching all config files. - --default <value>
-
When using
--get, and the requested variable is not found, behave as if <value> were the value assigned to the that variable.
CONFIGURATION
pager.config is only respected when listing configuration, i.e., when
using --list or any of the --get-* which may return multiple results.
The default is to use a pager.
FILES
If not set explicitly with --file, there are four files where
git config will search for configuration options:
- $(prefix)/etc/gitconfig
-
System-wide configuration file.
- $XDG_CONFIG_HOME/git/config
-
Second user-specific configuration file. If $XDG_CONFIG_HOME is not set or empty,
$HOME/.config/git/configwill be used. Any single-valued variable set in this file will be overwritten by whatever is in~/.gitconfig. It is a good idea not to create this file if you sometimes use older versions of Git, as support for this file was added fairly recently. - ~/.gitconfig
-
User-specific configuration file. Also called "global" configuration file.
- $GIT_DIR/config
-
Repository specific configuration file.
- $GIT_DIR/config.worktree
-
This is optional and is only searched when
extensions.worktreeConfigis present in $GIT_DIR/config.
If no further options are given, all reading options will read all of these files that are available. If the global or the system-wide configuration file are not available they will be ignored. If the repository configuration file is not available or readable, git config will exit with a non-zero error code. However, in neither case will an error message be issued.
The files are read in the order given above, with last value found taking precedence over values read earlier. When multiple values are taken then all values of a key from all files will be used.
You may override individual configuration parameters when running any git
command by using the -c option. See git[1] for details.
All writing options will per default write to the repository specific
configuration file. Note that this also affects options like --replace-all
and --unset. git config will only ever change one file at a time.
You can override these rules either by command-line options or by environment
variables. The --global, --system and --worktree options will limit
the file used to the global, system-wide or per-worktree file respectively.
The GIT_CONFIG environment variable has a similar effect, but you
can specify any filename you want.
ENVIRONMENT
- GIT_CONFIG
-
Take the configuration from the given file instead of .git/config. Using the "--global" option forces this to ~/.gitconfig. Using the "--system" option forces this to $(prefix)/etc/gitconfig.
- GIT_CONFIG_NOSYSTEM
-
Whether to skip reading settings from the system-wide $(prefix)/etc/gitconfig file. See git[1] for details.
See also FILES.
- GIT_CONFIG_COUNT
- GIT_CONFIG_KEY_<n>
- GIT_CONFIG_VALUE_<n>
-
If GIT_CONFIG_COUNT is set to a positive number, all environment pairs GIT_CONFIG_KEY_<n> and GIT_CONFIG_VALUE_<n> up to that number will be added to the process’s runtime configuration. The config pairs are zero-indexed. Any missing key or value is treated as an error. An empty GIT_CONFIG_COUNT is treated the same as GIT_CONFIG_COUNT=0, namely no pairs are processed. These environment variables will override values in configuration files, but will be overridden by any explicit options passed via
git-c.This is useful for cases where you want to spawn multiple git commands with a common configuration but cannot depend on a configuration file, for example when writing scripts.
EXAMPLES
Given a .git/config like this:
# # This is the config file, and # a '#' or ';' character indicates # a comment # ; core variables [core] ; Don't trust file modes filemode = false ; Our diff algorithm [diff] external = /usr/local/bin/diff-wrapper renames = true ; Proxy settings [core] gitproxy=proxy-command for kernel.org gitproxy=default-proxy ; for all the rest ; HTTP [http] sslVerify [http "https://weak.example.com"] sslVerify = false cookieFile = /tmp/cookie.txt
you can set the filemode to true with
% git config core.filemode true
The hypothetical proxy command entries actually have a postfix to discern what URL they apply to. Here is how to change the entry for kernel.org to "ssh".
% git config core.gitproxy '"ssh" for kernel.org' 'for kernel.org$'
This makes sure that only the key/value pair for kernel.org is replaced.
To delete the entry for renames, do
% git config --unset diff.renames
If you want to delete an entry for a multivar (like core.gitproxy above), you have to provide a regex matching the value of exactly one line.
To query the value for a given key, do
% git config --get core.filemode
or
% git config core.filemode
or, to query a multivar:
% git config --get core.gitproxy "for kernel.org$"
If you want to know all the values for a multivar, do:
% git config --get-all core.gitproxy
If you like to live dangerously, you can replace all core.gitproxy by a new one with
% git config --replace-all core.gitproxy ssh
However, if you really only want to replace the line for the default proxy, i.e. the one without a "for …" postfix, do something like this:
% git config core.gitproxy ssh '! for '
To actually match only values with an exclamation mark, you have to
% git config section.key value '[!]'
To add a new proxy, without altering any of the existing ones, use
% git config --add core.gitproxy '"proxy-command" for example.com'
An example to use customized color from the configuration in your script:
#!/bin/sh
WS=$(git config --get-color color.diff.whitespace "blue reverse")
RESET=$(git config --get-color "" "reset")
echo "${WS}your whitespace color or blue reverse${RESET}"
For URLs in https://weak.example.com, http.sslVerify is set to
false, while it is set to true for all others:
% git config --type=bool --get-urlmatch http.sslverify https://good.example.com true % git config --type=bool --get-urlmatch http.sslverify https://weak.example.com false % git config --get-urlmatch http https://weak.example.com http.cookieFile /tmp/cookie.txt http.sslverify false
CONFIGURATION FILE
The Git configuration file contains a number of variables that affect
the Git commands' behavior. The files .git/config and optionally
config.worktree (see the "CONFIGURATION FILE" section of
git-worktree[1]) in each repository are used to store the
configuration for that repository, and $HOME/.gitconfig is used to
store a per-user configuration as fallback values for the .git/config
file. The file /etc/gitconfig can be used to store a system-wide
default configuration.
The configuration variables are used by both the Git plumbing
and the porcelains. The variables are divided into sections, wherein
the fully qualified variable name of the variable itself is the last
dot-separated segment and the section name is everything before the last
dot. The variable names are case-insensitive, allow only alphanumeric
characters and -, and must start with an alphabetic character. Some
variables may appear multiple times; we say then that the variable is
multivalued.
Syntax
The syntax is fairly flexible and permissive; whitespaces are mostly ignored. The # and ; characters begin comments to the end of line, blank lines are ignored.
The file consists of sections and variables. A section begins with
the name of the section in square brackets and continues until the next
section begins. Section names are case-insensitive. Only alphanumeric
characters, - and . are allowed in section names. Each variable
must belong to some section, which means that there must be a section
header before the first setting of a variable.
Sections can be further divided into subsections. To begin a subsection put its name in double quotes, separated by space from the section name, in the section header, like in the example below:
[section "subsection"]
Subsection names are case sensitive and can contain any characters except
newline and the null byte. Doublequote " and backslash can be included
by escaping them as \" and \\, respectively. Backslashes preceding
other characters are dropped when reading; for example, \t is read as
t and \0 is read as 0 Section headers cannot span multiple lines.
Variables may belong directly to a section or to a given subsection. You
can have [section] if you have [section "subsection"], but you don’t
need to.
There is also a deprecated [section.subsection] syntax. With this
syntax, the subsection name is converted to lower-case and is also
compared case sensitively. These subsection names follow the same
restrictions as section names.
All the other lines (and the remainder of the line after the section
header) are recognized as setting variables, in the form
name = value (or just name, which is a short-hand to say that
the variable is the boolean "true").
The variable names are case-insensitive, allow only alphanumeric characters
and -, and must start with an alphabetic character.
A line that defines a value can be continued to the next line by ending it with a \; the backslash and the end-of-line are stripped. Leading whitespaces after name =, the remainder of the line after the first comment character # or ;, and trailing whitespaces of the line are discarded unless they are enclosed in double quotes. Internal whitespaces within the value are retained verbatim.
Inside double quotes, double quote " and backslash \ characters
must be escaped: use \" for " and \\ for \.
The following escape sequences (beside \" and \\) are recognized: \n for newline character (NL), \t for horizontal tabulation (HT, TAB) and \b for backspace (BS). Other char escape sequences (including octal escape sequences) are invalid.
Includes
The include and includeIf sections allow you to include config
directives from another source. These sections behave identically to
each other with the exception that includeIf sections may be ignored
if their condition does not evaluate to true; see "Conditional includes"
below.
You can include a config file from another by setting the special
include.path (or includeIf.*.path) variable to the name of the file
to be included. The variable takes a pathname as its value, and is
subject to tilde expansion. These variables can be given multiple times.
The contents of the included file are inserted immediately, as if they had been found at the location of the include directive. If the value of the variable is a relative path, the path is considered to be relative to the configuration file in which the include directive was found. See below for examples.
Conditional includes
You can include a config file from another conditionally by setting a
includeIf.<condition>.path variable to the name of the file to be
included.
The condition starts with a keyword followed by a colon and some data whose format and meaning depends on the keyword. Supported keywords are:
gitdir-
The data that follows the keyword
gitdir:is used as a glob pattern. If the location of the .git directory matches the pattern, the include condition is met.The .git location may be auto-discovered, or come from
$GIT_DIRenvironment variable. If the repository is auto discovered via a .git file (e.g. from submodules, or a linked worktree), the .git location would be the final location where the .git directory is, not where the .git file is.The pattern can contain standard globbing wildcards and two additional ones,
**/and/**, that can match multiple path components. Please refer to gitignore[5] for details. For convenience:-
If the pattern starts with
~/,~will be substituted with the content of the environment variableHOME. -
If the pattern starts with
./, it is replaced with the directory containing the current config file. -
If the pattern does not start with either
~/,./or/,**/will be automatically prepended. For example, the patternfoo/barbecomes**/foo/barand would match/any/path/to/foo/bar. -
If the pattern ends with
/,**will be automatically added. For example, the patternfoo/becomesfoo/**. In other words, it matches "foo" and everything inside, recursively.
-
gitdir/i-
This is the same as
gitdirexcept that matching is done case-insensitively (e.g. on case-insensitive file systems) onbranch-
The data that follows the keyword
onbranch:is taken to be a pattern with standard globbing wildcards and two additional ones,**/and/**, that can match multiple path components. If we are in a worktree where the name of the branch that is currently checked out matches the pattern, the include condition is met.If the pattern ends with
/,**will be automatically added. For example, the patternfoo/becomesfoo/**. In other words, it matches all branches that begin withfoo/. This is useful if your branches are organized hierarchically and you would like to apply a configuration to all the branches in that hierarchy.
A few more notes on matching via